#!/usr/bin/php -q
<?php

/* vim: set expandtab tabstop=4 shiftwidth=4 foldmethod=marker: */
// $HeadURL$
// +------------------------------------------------------------------------+
// | Maintain - DNS/DHCP Managment Tool                                     |
// +------------------------------------------------------------------------+
// | Copyright (c) 2004 Open Source University                              |
// +------------------------------------------------------------------------+
// | This program is free software; you can redistribute it and/or          |
// | modify it under the terms of the GNU General Public License            |
// | as published by the Free Software Foundation; either version 2         |
// | of the License, or (at your option) any later version.                 |
// |                                                                        |
// | This program is distributed in the hope that it will be useful,        |
// | but WITHOUT ANY WARRANTY; without even the implied warranty of         |
// | MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          |
// | GNU General Public License for more details.                           |
// |                                                                        |
// | You should have received a copy of the GNU General Public License      |
// | along with this program; if not, write to the Free Software            |
// | Foundation, Inc., 59 Temple Place - Suite 330,                         |
// | Boston, MA  02111-1307, USA.                                           |
// +------------------------------------------------------------------------+
// | Author(s): Daniel Robert <robert@osuosl.org>                           |
// +------------------------------------------------------------------------+
// $Id$

/** This is a CLI script to create the base DB_DataObject classes in the
  * proper path with the proper options.
  *
  * Additionally, this will rename the maintain.links.ini file appropriately,
  * based on the name of the database
  *
  * @package    bin
  */

require_once dirname(__FILE__).'/includes.php';
require_once CLASSDIR."/DAOWriter.php";

// setup the command-line parameters config array
$validargs = array(
        'verbose' => array( 'short' => 'v',
                        'min' => 0,
                        'max' => 1,
                        'default' => 1,
                        'desc' => _("Set the verbosity level.  Valid range is 0-5")
                ),
        'link' => array( 'short' => 'l',
                        'min' => 0,
                        'max' => 1,
                        'default' => 0,
                        'desc' => _("Do not make DB_DataObject files, just make a symlink "
                            ."pointing to the default INI file. Use this if you did not "
                            ."change the database structure, only its name.")
                ),
        );

// initialize the executable
$bin = new MaintainBin($validargs);

// parse the command-line
$args = $bin->parseArgs();

// set the verbosity variable
$verbose = ($args->isDefined('v')) ? $args->getValue('v') : 1;

// make sure verbosity level is within reason
if ($verbose > 5) {
    $msg = _("Verbosity setting must be between 0 and 5") . "\n";
    $fullmsg = Console_Getargs::getHelp($validargs, $msg);
    $bin->fail($fullmsg);
}

// if symlink only, do it now and exit
if ($args->isDefined('l')) {
    if (!isset($conf['database']['database'])) {
        $bin->fail(_("Fatal error: The Maintain database setup in the config file is invalid."));
    }
    
    $linkname = $conf['database']['database'].".links.ini";
    $createsymlink = @symlink("maintain.links.ini.dist", DAODIR."/".$linkname);
    if (!$createsymlink) {
        $bin->fail(_("Error: Creating the symbolic link failed. A file named ".$linkname." might already exist, or the directory ".DAODIR." is not accessible."));
    }
    $bin->finish(_("Symbolic link to database config file successfully created."));
}
// otherwise, start to write all DAO files including the ini file.

// initialize the DAOWriter
$writer = new DAOWriter();

// skip the validation, write the DAOs
$writer->createTables(true, $verbose);

if ($writer->getStatus() != OK) {
    $bin->fail($writer->getFormattedMessages('txt'));
}

//rename <dbname>.lins.ini file to match db name
if (!$writer->verifyLinksIni())
    print $writer->getFormattedMessages('txt');

// if verbosity is at 0, exit silently
$bin->finish();

?>
