#!/usr/bin/php -q
<?php

/* vim: set expandtab tabstop=4 shiftwidth=4 foldmethod=marker: */
// $HeadURL$
// +------------------------------------------------------------------------+
// | Maintain - DNS/DHCP Managment Tool                                     |
// +------------------------------------------------------------------------+
// | Copyright (c) 2004 Open Source University                              |
// +------------------------------------------------------------------------+
// | This program is free software; you can redistribute it and/or          |
// | modify it under the terms of the GNU General Public License            |
// | as published by the Free Software Foundation; either version 2         |
// | of the License, or (at your option) any later version.                 |
// |                                                                        |
// | This program is distributed in the hope that it will be useful,        |
// | but WITHOUT ANY WARRANTY; without even the implied warranty of         |
// | MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          |
// | GNU General Public License for more details.                           |
// |                                                                        |
// | You should have received a copy of the GNU General Public License      |
// | along with this program; if not, write to the Free Software            |
// | Foundation, Inc., 59 Temple Place - Suite 330,                         |
// | Boston, MA  02111-1307, USA.                                           |
// +------------------------------------------------------------------------+
// | Author(s): Daniel Robert <robert@osuosl.org>                           |
// +------------------------------------------------------------------------+
// $Id: modules.php 4642 2006-07-24 17:08:43Z claym $


/**
  * The Maintain tool for installing/uninstalling modules
  *
  * @author $Author$
  * @package    bin
  */

require_once dirname(__FILE__).'/includes.php';

// we're going to need the Module class
require_once MODDIR.'/module.php';

/* Setup the array of command-line values */
$validargs = array(
        'list|show' => array( 'short' => 'l|s',
                    'min' => 0,
                    'max' => 1,
                    'default' => '',
                    'desc' => _("Display all installed modules.") .
                        _("Optional arguments are 'enabled'/'disabled'")
                ),
        'install' => array( 'short' => 'i',
                    'min' => 1,
                    'max' => 1,
                    'desc' => _("Install a module by name")
                ),
        'uninstall|remove' => array( 'short' => 'u|r',
                    'min' => 1,
                    'max' => 1,
                    'desc' => _("Uninstall a module by name")
                ),
        'upgrade|update' => array ( 'short' => 'g',
                    'min' => 1,
                    'max' => 1,
                    'desc' => _("Upgrade a module by name")
                ),
        'enable' => array( 'short' => 'e',
                    'min' => 1,
                    'max' => 1,
                    'desc' => _("Enable a module by name")
                ),
        'disable' => array( 'short' => 'd',
                    'min' => 1,
                    'max' => 1,
                    'desc' => _("Disable a module by name")
                ),
        'force' => array ( 'short' => 'f',
                    'max' => 0,
                    'desc' => _("Force enabling/disabling as well as installing/uninstalling modules")
                )
        );

// set up the executable
$bin = new MaintainBin($validargs);

// parse the input
$args = $bin->parseArgs();

// is action forced?
$force = $args->isDefined('force');

// {{{ install
// if we're installing, perform the install
if ($args->isDefined('i')) {

    $mod = $args->getValue('i');
    
    // action forced?
    if ($force) {
        $result = $bin->confirm(_("You are about to force the installation of the module")."\n".$mod."\n"
            ._("Do you really want to proceed?"));
        if (!$result) $bin->fail(_("Forced installation aborted."));
    }

    $success = Module::install($mod, $force);

    if (PEAR::isError($success)) {
        $bin->fail($success->getMessage());

    } elseif (!$success) {
        $bin->fail(_("Could not install") . " $mod");
    }

    $msg = "$mod " . _("successfully installed");
    $bin->finish($msg);

// }}}
// {{{ uninstall
} elseif ($args->isDefined('u')) {

    $mod = $args->getValue('u');
    
    // action forced?
    if ($force) {
        $result = $bin->confirm(_("You are about to force uninstalling the module")."\n".$mod."\n"
            ._("Do you really want to proceed?"));
        if (!$result) $bin->fail(_("Forced uninstall aborted."));
    }

    $success = Module::uninstall($mod, $force);

    // if error response, fail
    if (PEAR::isError($success)) {
        $bin->fail($success->getMessage());

    // if uninstall failed, fail
    } elseif(!$success) {
        $bin->fail(_("Could not uninstall") . " $mod");
    }

    $msg = $args->getValue('u') . ' ' . _("successfully removed");
    $bin->finish($msg);

// }}}
// {{{ upgrade
} elseif ($args->isDefined('upgrade')) {

    $mod = $args->getValue('upgrade');
    $success = Module::upgrade($mod);

    // if error response, fail
    if (PEAR::isError($success)) {
        $bin->fail($success->getMessage());

    // if upgrade failed, fail
    } elseif (!$success) {
        $bin->fail(_("Could not upgrade") . " $mod");
    }

    $msg = $args->getValue('upgrade') . ' ' . _("successfully upgraded");
    $bin->finish($msg);
// }}}
// {{{ list
} elseif ($args->isDefined('l')) {

    $list = $args->getValue('l');

    // grab a list of all modules in the database
    $modules = DB_DataObject::Factory('modules');

    if ($list == 'enabled') {
        $modules->enabled = 1;

    } elseif ($list == 'disabled') {
        $modules->enabled = 0;

    }

    // if no modules were found, inform the user as much
    if (!$modules->find()) {
        $msg = _("No modules are currently") . ' ';
        if (!$list) {
            $bin->finish($msg . _("installed"));

        }

        $bin->finish($msg . $list);

    } // no matching modules

    $msg = ($list) ? ucfirst($list).' ' . _("modules:") : _("Modules:");
    $msg .= "\n";

    // add the result to an array
    $modlist = array($msg);
    while ($modules->fetch()) {

        $line = ucfirst($modules->name);

        if (!$list) {
            $line .= ' (';
            $line .= ($modules->enabled) ? _('enabled') : _('disabled');
            $line .= ")";
        }

        $modlist[] = "$line";
    }

    // comma-separate the list
    $bin->finish(implode("\n", $modlist));

// }}}
// {{{ enable
} elseif ($args->isDefined('e')) {

    $mod = $args->getValue('e');

    if (!$mod) {
        $bin->fail(Console_Getargs::getHelp($validargs, 
                _("You must specify the module to enable")."\n"));
    }

    // action forced?
    if ($force) {
        $result = $bin->confirm(_("You are about to force enabling the module")."\n".$mod."\n"
            ._("Do you really want to proceed?"));
        if (!$result) $bin->fail(_("Forced enable aborted."));
    }

    $success = Module::enable($args->getValue('e'), $force);

    if (PEAR::isError($success)) {
        $bin->fail($success);
    } elseif (!$success) {
        $bin->fail(_("Could not enable") . " $mod");
    }

    // try to instanciate module. Config file will be checked
    $newmodule = new $mod();

    if ($newmodule->hasConfigErrors) {
        $bin->fail(_("Could not enable") . " $mod\n\n" . _('An error message was emailed to the administrator(s).'));
    } else {
        $bin->finish($args->getValue('e') . ' ' . _("successfully enabled"));
    }

// }}}
// {{{ disable
} elseif ($args->isDefined('d')) {

    $mod = $args->getValue('d');

    if (!$mod) {
        $bin->fail(Console_Getargs::getHelp($validargs, 
                _("You must specify the module to disable")."\n"));
    }

    // action forced?
    if ($force) {
        $result = $bin->confirm(_("You are about to force disabling the module")."\n".$mod."\n"
            ._("Do you really want to proceed?"));
        if (!$result) $bin->fail(_("Forced disable aborted."));
    }

    $success = Module::disable($args->getValue('d'), $force);

    if (PEAR::isError($success)) {
        $bin->fail($success);
    } elseif (!$success) {
        $bin->fail(_("Could not disable") . " $mod");
    }

    $bin->finish("$mod " . _("successfully disabled"));
// }}}
// no options were given
} else {

    $bin->fail(Console_Getargs::getHelp($validargs));
}

?>
