<?php

/* vim: set expandtab tabstop=4 shiftwidth=4 foldmethod=marker: */
// $HeadURL$
// +------------------------------------------------------------------------+
// | Maintain - DNS/DHCP Managment Tool                                     |
// +------------------------------------------------------------------------+
// | Copyright (c) 2004 Open Source University                              |
// +------------------------------------------------------------------------+
// | This program is free software; you can redistribute it and/or          |
// | modify it under the terms of the GNU General Public License            |
// | as published by the Free Software Foundation; either version 2         |
// | of the License, or (at your option) any later version.                 |
// |                                                                        |
// | This program is distributed in the hope that it will be useful,        |
// | but WITHOUT ANY WARRANTY; without even the implied warranty of         |
// | MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          |
// | GNU General Public License for more details.                           |
// |                                                                        |
// | You should have received a copy of the GNU General Public License      |
// | along with this program; if not, write to the Free Software            |
// | Foundation, Inc., 59 Temple Place - Suite 330,                         |
// | Boston, MA  02111-1307, USA.                                           |
// +------------------------------------------------------------------------+
// | Author(s): Zack Bartel <bartel@osuosl.org>                             |
// +------------------------------------------------------------------------+
// $Id: DomainEditForm.php 3971 2005-12-15 23:09:25Z bartel $

/**
 *  DomainEditForm.php - contains the {@link DomainEditForm} class
 *
 *  @author $Author:$
 *  @package    htdocs  
 *  @subpackage forms
 *  @filesource
 */

require_once(FORMDIR."/DomainCreateForm.php");

/**
 *  This class is the form that is used to edit a Domain object
 *
 *  @package    htdocs
 *  @subpackage forms
 */ 
class DomainEditForm extends DomainCreateForm
{

//{{{   vars

    /**
     *  This array maps form labels to DBDO vars and vice verse
     */
    var $map = array("Domain_Name"=>"name",
                    /*"Master_Domain_Name"=>"master_domain",*/
                    "Enabled"=>"enabled",
                    "Comment"=>"comment");

//}}}

//{{{   form_init_elements()

    /**
     *  This method is responsible for intializing the form elements that are going
     *  to be displayed and filled in by the user.
     */
    function form_init_elements() 
    {
        $name = new FEText(_("Domain Name"), true, 47);
        /* Don't think this should be here */
        //$parent = new FEListBox(_("Master Domain Name"), false);
        $status = new FEListBox(_("Enabled"), false);
        $comment = new FETextArea(_("Comment"),true, 5, 40);
        
        //$parent->set_list_data($this->_domains()); 
        $status->set_list_data(array("Enabled"=>1,
                                    "Disabled"=>0));

        $this->add_elements($name, $comment, /*$parent,*/ $status);
    }

//}}}
//{{{   form_init_data()
    /*
     *  Function that initializes the form elements to the
     *  existing values.
     *
     *
     */

    function form_init_data()
    {
        $dom = new Domain($_REQUEST['id']);
        $dom->query("BEGIN");
        $formarray = $this->remap($dom->toArray(), FORM);

        foreach($formarray as $var=>$val)
            $this->set_element_value($var, $val);

        $this->set_element_value(_("Comment"), _("Domain Updated"));
        $dom->query("COMMIT");

    }
//}}}
//{{{   form_action()

    function form_action()
    {
        global $user;
        $res = true;

        switch(strtolower($this->get_action())) {

        case 'submit':
            $vals = $this->get_all_element_values();
        
            $domain = new Domain($_REQUEST['id']);
            $domain->query("BEGIN");

            $domain->setFrom($this->remap($vals, DBDO));

            if($domain->update() === false) {   //Returns 0 rows affected when not updating anything
                $domain->query("ROLLBACK");
                $res = false;
                $this->add_error(_("ERROR"),_("Couldn't update database"));
            } else {
                $domain->query("COMMIT");
                $domain->write_history($vals[_("Comment")], ACTION_MODIFY,$user->username);
            }
            break;

        default:
            die_hard(_("Invalid form action"));
            break;
        }
        return $res;
    }

//}}}i
//{{{   form_success()

    function form_success()
    {
        $div = html_div("msuccess");
        $indlink = html_a(web_root()."index.php", _("[Return to Index]"));
        $sublink = html_a(web_root()."domain.php",_("[Return to Domain List]"));
        $div->add(_("Domain successfully modified!"),html_br());
        $div->add($indlink, $sublink);
        
       // header("Location: ".$_SERVER['SCRIPT_NAME']);
        return $div;
    }

//}}}

//{{{   form_hidden()

    /**
     *  Add some hidden values that we'll need later.  The ID of the OPTION and the
     *  get action so we can get back to the form.
     *
     *  @return Container   Container that has the hidden elements in it
     */
    function form_hidden()
    {
        $action =  new FEHidden(_("action"), $_REQUEST['action']);
        $id = new FEHidden(_("id"), $_REQUEST['id']);

        $this->add_elements($action, $id);

        return container($this->element_form(_('action')), $this->element_form(_('id')));
    }

//}}}
//{{{   _domains()

    /**
     *  This is a helper method that is used to populate the parent domains listbox
     *
     *  @return Array   $domains  An array of domains indexed by name
     */
    function _domains()
    {
        $domains = array();
        
        $domain = DB_DataObject::Factory('domain');
        $domain->query("BEGIN");
        $domain->orderBy("name ASC");
        $domain->find();
        /* If we want to be the TLD */
        $domains["none (Parent)"] = 0;
        
        while($domain->fetch())
            $domains[$domain->name] = $domain->id;
        $domain->query("COMMIT");
        return $domains;
    }

//}}}

}

?>
