<?php
/* vim: set expandtab tabstop=4 shiftwidth=4 foldmethod=marker: */
// $HeadURL: https://svn.osuosl.org/public/maintain-project/trunk/class/htdocs/forms/LoginForm.php $
// +------------------------------------------------------------------------+
// | Maintain - DNS/DHCP Managment Tool                                     |
// +------------------------------------------------------------------------+
// | Copyright (c) 2004 Open Source University                              |
// +------------------------------------------------------------------------+
// | This program is free software; you can redistribute it and/or          |
// | modify it under the terms of the GNU General Public License            |
// | as published by the Free Software Foundation; either version 2         |
// | of the License, or (at your option) any later version.                 |
// |                                                                        |
// | This program is distributed in the hope that it will be useful,        |
// | but WITHOUT ANY WARRANTY; without even the implied warranty of         |
// | MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          |
// | GNU General Public License for more details.                           |
// |                                                                        |
// | You should have received a copy of the GNU General Public License      |
// | along with this program; if not, write to the Free Software            |
// | Foundation, Inc., 59 Temple Place - Suite 330,                         |
// | Boston, MA  02111-1307, USA.                                           |
// +------------------------------------------------------------------------+
// | Author(s): Michael Clay <claym@osuosl.org>                             |
// +------------------------------------------------------------------------+
// $Id: LoginForm.php 4831 2006-09-19 23:40:27Z morgabra $

require_once(INCLUDEDIR."/auth.php");
require_once(HTDOCS."/includes.php");

/**
 *  This class is the login form for, you guessed it, logging into the system.
 *  It extends the glorious phpHtmlLib class FormContent and is to be shoved in
 *  a form processor for use.
 */
class LoginForm extends FormContent
{

//{{{   vars

    var $error; //the error message if any.

//}}}
//{{{   form_init_elements()

    /**
     *  This method adds the form elements to the form content object
     */
    function form_init_elements()
    {
        $pass = new LoginPassword(_("Password"),true);
        $user = new FEText(_("Username"),true);
        
        //Remove the asterisk from the label of required elements and add to form
        $this->set_required_marker(null);
        $this->add_element($pass);
        $this->add_element($user);
        $this->set_action($_SERVER["SCRIPT_NAME"]);

        $ary = $_SERVER['REQUEST_URI'];
        $ary = split('/', $ary);
        $path = $ary[count($ary)-1];
        $ary = split('login.php\?', $path);
        $path = $ary[count($ary)-1];

        $hidden = new FEHidden("path", $_SERVER['REQUEST_URI']);
        $this->add_element($hidden);
        
    }

//}}}
//{{{   form()

    /**
     *  This method is used to lay out how the form is displayed when the page
     *  is rendered
     *
     *  @return Container   $div    a phphtmllib div container
     */
    function form()
    {
        global $images;
        $div = container();
        
        $div->add($this->element_form("path"));
        
        /* Login form table */
        $table = html_table();
        $table->set_class("login");
        $login = html_table();
        $welcome = html_span("header1");
        $welcome->add(_("Welcome to Maintain"));
        $login->add($welcome, html_br());
        $login->add_row($this->element_label(_("Username")),$this->element_form(_("Username")));
        $login->add_row($this->element_label(_("Password")),$this->element_form(_("Password")));
        $login->add_row("&nbsp",$this->add_action(_("Login")));
        $right = html_td();
        $right->add($login);

        $left = html_td();
        $left->add(html_img_href("login.php", $images[IMG_LOGIN]));      
        $table->add($left);
        $table->add($right);

        $div->add($table);
        $div->add("<script type='text/javascript'>document.getElementById('loginform_"._('Username')."').focus();</script>");
        return $div; 
    }

//}}}
//{{{   form_init_data()

    /**
     *  This method fills in the initial values for the form
     */
    function form_init_data()
    {
        $this->set_element_value(_("Password"), ""); //make sure the password field has been cleared
    }

//}}}
//{{{   form_backend_validation()

    /**
     *  This method handles validation of the form and the actual act of logging
     *  into the system.
     *
     *  @return boolean  $res    was validation successful?
     */
    function form_backend_validation()
    {
        global $conf;

        $res = false;

        if($conf['auth']['debug']) { //automatically a winner!
            session_start();
            $_SESSION['_authsession']['username'] = $_POST[_('Username')];
            $res = true;
        }
        else {
            $this->error = authenticate($_POST[_('Username')], $_POST[_('Password')]);
             //if there is no error you win
            if($this->error === true) {
                $res = true;
            } else {
                $this->add_error(_("ERROR"),$this->error); //tell them what's wrong
            }
        }
        return $res;   
    }

//}}}
//{{{   form_action()
    
    /**
     *  This method performs the act of setting the user's zone and shoving them
     *  to index.
     *
     *  @return boolean true action was successfully performed
     */
    function form_action()
    {
        global $conf;
        global $user;
        
        $ary = $_REQUEST['path'];
        $ary = split('/', $ary);
        $path = $ary[count($ary)-1];
        $ary = split('login.php\?', $path);
        $path = $ary[count($ary)-1];

        $user = new User($_SESSION['_authsession']['username'], LOGIN);

        $_SESSION['zone'] = $user->preferred_zone; //set their initial zone
        
        session_write_close();

        if($path == 'login.php') {
            header("Location: ".Module::apply_filter('post_login_forward', web_root()."index.php"));
        }
        else {
            // forward me to the index page (no matter if we are in a zone or not)
            header("Location: ".Module::apply_filter('post_login_forward', web_root()."$path"));
        }
        return true;
    }

//}}}

}

/**
 *  This is a cheesy extentions of the phpHtmlLib FEPassword class.  This class
 *  needed to be extended in order to avoid it using phpHtmlLib's built in password
 *  validation, which doesn't fit the Maintain password scheme.
 */
class LoginPassword extends FEPassword
{

//{{{   validate()

    /**
     *  This method would usually be used to validate this particular form element.
     *  It now does absolutely nothing but return true since the validation for
     *  this element is now going to be handled in the form itself inside the
     *  form_backend_validation() method.
     *
     *  @return boolean true    validation successful
     */
    function validate(&$_FormValidation)
    {
        return true;
    }

//}}}

}

?>
