<?php

/* vim: set expandtab tabstop=4 shiftwidth=4 foldmethod=marker: */
// $HeadURL$
// +------------------------------------------------------------------------+
// | Maintain - DNS/DHCP Managment Tool                                     |
// +------------------------------------------------------------------------+
// | Copyright (c) 2004 Open Source University                              |
// +------------------------------------------------------------------------+
// | This program is free software; you can redistribute it and/or          |
// | modify it under the terms of the GNU General Public License            |
// | as published by the Free Software Foundation; either version 2         |
// | of the License, or (at your option) any later version.                 |
// |                                                                        |
// | This program is distributed in the hope that it will be useful,        |
// | but WITHOUT ANY WARRANTY; without even the implied warranty of         |
// | MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          |
// | GNU General Public License for more details.                           |
// |                                                                        |
// | You should have received a copy of the GNU General Public License      |
// | along with this program; if not, write to the Free Software            |
// | Foundation, Inc., 59 Temple Place - Suite 330,                         |
// | Boston, MA  02111-1307, USA.                                           |
// +------------------------------------------------------------------------+
// | Author(s): Zack Bartel <bartel@osuosl.org>                             |
// |            Frederic Wenzel <fred@osuosl.org>                           |
// +------------------------------------------------------------------------+
// $Id: ObjectOptionCreateForm.php 3964 2006-01-05 21:49:51Z bartel $

/**
 *  ObjectOptionCreateForm.php - contains the {@link HelpCreateForm} class
 *                               as well as a simple MaintainDeleteConfirm
 *                               overrriden class that takes care of delete
 *  @author $Author$
 *  @package    htdocs
 *  @subpackage forms
 *  @filesource
 */

require_once(FORMDIR."/MaintainForm.php");

/**
 *  This class is used for creating ObjectOption Records
 *  
 *  @package    htdocs
 *  @subpackage forms
 */
class ObjectOptionCreateForm extends MaintainForm
{

//{{{   vars
   
    /**
     *  Dirty cheat to map form elements to dbdo vars
     */
    var $map = array("Option_Value"=>"value",
                    "DHCP_Option"=>"dhcp_option",
                    "Scope"=>"scope");
    
//}}}

//{{{   form_init_elements()

    /**
     *  This method initializes all of the elements of the form
     */
    function form_init_elements()
    {
        $value = new FEText(_("Option Value"), true);
        $option = new FEListBox(_("DHCP Option"));
        $option->set_list_data($this->get_options());
        $scope = new FEListBox(_("Scope"));

        //Set the available scopes
        if ($_REQUEST['object_type'] == 'workgroup') 
            $scope->set_list_data(array("Workgroup"=>"workgroup", "Host"=>"host"));
        else if ($_REQUEST['object_type'] == 'subnet')
            $scope->set_list_data(array("Subnet" => "subnet", "Host"=>"host"));
        else if ($_REQUEST['object_type'] == 'host')
            $scope->set_list_data(array("Host"=>"host"));
        else
            $scope->set_list_data(array("Global"=>"global"));

        $this->add_elements($value, $option, $scope);
        $this->set_action($_SERVER["SCRIPT_NAME"]);
        if(isset($_REQUEST['object_type']) && strcasecmp($_REQUEST['object_type'],'global'))
            $this->set_cancel_loc(web_root().$_REQUEST['object_type'].".php");
        else
            $this->set_cancel_loc(web_root().'admin.php');
        
    }

//}}}
//{{{   form_backend_validation()

    /**
     *  This method validates the values that were given before processing.
     *  Since all help documents are valid as long as neither field is null
     *  I think the form does all of the work.
     *
     *  @return Boolean     True if data is valid
     */
    function form_backend_validation()
    {
        $result = true;
        $vals = $this->remap($this->get_all_element_values(), DBDO);
        $opt = new Option();
        $opt->setFrom($vals);
        $opt->object_id = $_REQUEST['object_id'];
        $opt->type = $_REQUEST['object_type'];
        $valid = $opt->validate();
        if ($valid !== true)    {
            $errors = $this->remap($valid, FORM);
            foreach ($errors as $var=>$val)   {
                if ($val !== true)   {
                    $this->add_error($var, $val);
                    $result = false;
                }
            }
        } 
        return $result;
        
    }

//}}}
//{{{   form_action()

    /**
     *  This method is what processes the data after validation.  In this case
     *  it creates a ObjectOption record.
     *
     *  @return Boolean     True if action was successful
     */
    function form_action()
    {
        global $user;
    
        $res = true;
        switch(strtolower($this->get_action())) {
        
        case _("submit"):
            /* Grab form data and turn it into DB_DO dudes */
            $vars = $this->remap($this->get_all_element_values(), DBDO);

            $opt = new Option();
            $opt->query("BEGIN");
            $opt->setFrom($vars);
            $opt->object_id = $_REQUEST['object_id'];
            $opt->type = $_REQUEST['object_type'];
            $opt->insert();
            $opt->query("COMMIT");
            break;

        default:
            $this->add_error(_("ERROR"), _("Invalid form action"));
            break;
        
        }

        return $res;
    }

//}}}
//{{{   form_success()

    /**
     *  This method lets them know they were successful in creating the record
     *  and allows them to go back to the index or to the list.
     */
    function form_success()
    {
        $div = html_div("hsuccess");
        $type = $_REQUEST['object_type'];
        $object_id = $_REQUEST['object_id'];
        $indlink = html_a("index.php", "[Return to Home]");
        $link = html_a("option.php?object_type=$type&amp;object_id=$object_id", "[Return to $type List]");
        $div->add(container("Option Record created successfully!",html_br()));
        $div->add(container($indlink, $link));
        return $div; 
    }

//}}}
//{{{ get_options()
    /* Returns a list of valid options */

    function get_options()
    {
        $ary = array();
        $do = new DHCPOption();
        $do->find();
        while($do->fetch())
            $ary[$do->name] = $do->id;
        return $ary; 
    }
//}}}
//{{{   form_hidden()

    /**
     *  This method should be overloaded by the child class and is used to add the
     *  hidden form variables to the form.
     *
     *  @return FEHidden    A hidden form element
     */
    function form_hidden()
    {
        $hidden = new FEHidden("action",$_REQUEST['action']);
        $this->add_element($hidden);

        $objid = new FEHidden("object_id", $_REQUEST['object_id']);
        $this->add_element($objid);
        
        $objtype = new FEHidden("object_type", $_REQUEST['object_type']);
        $this->add_element($objtype);

        return container($this->element_form(_("action")), $this->element_form("object_id"), $this->element_form("object_type"));
    }

//}}}

}
?>
