<?php

/* vim: set expandtab tabstop=4 shiftwidth=4 foldmethod=marker: */
// $HeadURL$
// +------------------------------------------------------------------------+
// | Maintain - DNS/DHCP Managment Tool                                     |
// +------------------------------------------------------------------------+
// | Copyright (c) 2004 Open Source University                              |
// +------------------------------------------------------------------------+
// | This program is free software; you can redistribute it and/or          |
// | modify it under the terms of the GNU General Public License            |
// | as published by the Free Software Foundation; either version 2         |
// | of the License, or (at your option) any later version.                 |
// |                                                                        |
// | This program is distributed in the hope that it will be useful,        |
// | but WITHOUT ANY WARRANTY; without even the implied warranty of         |
// | MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          |
// | GNU General Public License for more details.                           |
// |                                                                        |
// | You should have received a copy of the GNU General Public License      |
// | along with this program; if not, write to the Free Software            |
// | Foundation, Inc., 59 Temple Place - Suite 330,                         |
// | Boston, MA  02111-1307, USA.                                           |
// +------------------------------------------------------------------------+
// | Author(s): Michael Clay <claym@osuosl.org>                             |
// +------------------------------------------------------------------------+
// $Id: ZoneCreateForm.php 4606 2006-07-12 22:41:37Z morgabra $

/**
 *  ZoneCreateForm - This file contains the {@link ZoneCreateForm} class
 *
 *  @author $Author:$
 *  @package    htdocs
 *  @subpackage forms
 *  @filesource
 */

require_once(FORMDIR."/MaintainForm.php");

/**
 *  This class contains the form that is used by the users to create zones
 *
 *  @package    htdocs
 *  @subpackage forms
 */
class ZoneCreateForm extends MaintainForm
{

/*Vars*/
//{{{   vars

    /**
     *  This array is used to map form labels to DBDO vars and vice versa
     */
    var $map = array("Name"=>"name",
                    "Short_Description"=>"description",
                    "Comment"=>"comment");

//}}}

/*Methods*/
//{{{   form_init_elements()

    /**
     *  This method initializes the form elements that are going to be used
     */
    function form_init_elements()
    {
        $name = new FEText(_("Name"), true);
        $desc = new FEText(_("Short Description"),true);
        $comment = new FETextArea(_("Comment"), true, 5, 40);

        $this->add_elements($name, $desc, $comment);
    }

//}}}
//{{{   form_backend_validation()

    /**
     *  This method takes care of validation before the information is processed by
     *  the action method.
     *
     *  @return Boolean     True if data is valid
     */
    function form_backend_validation()
    {
        $res = true;
    
        $zn = new Zone();
        $zn->setAll();
        $zn->setFrom($this->remap($this->get_all_element_values(), DBDO));
        $valid = $zn->validate();

        //if validation wasn't successful
        if($valid !== true) {
            
            $errors = $this->remap($valid, FORM);
            
            //Loop through the rrors
            foreach($errors as $label=>$error) {
                if($error !== true) //If the field failed
                    $this->add_error($label, $error); //report the error
            }
            
            $res = false;
            
        }
        
        return $res;
    }

//}}}
//{{{   form_action()

    /**
     *  This method processes the data after it passes validation
     *
     *  @return Boolean     True if processing was successful
     */
    function form_action()
    {

        global $user;
        $res = true;
    
        switch(strtolower($this->get_action())) {
        
        case 'submit':
            $zn = new Zone(); //make a blank zone
            $zn->query("BEGIN");
            $zn->setAll(); //init defaults
            
            //Use form values
            $zn->setFrom($this->remap($this->get_all_element_values(), DBDO));
            
            //try to insert, if it fails
            if(!$zn->insert()) {
                $zn->query("ROLLBACK");
                $this->add_error(_("ERROR"),_("The zone could not be added to the database"));
                $res = false; //flag error
            } else {
                $zn->query("COMMIT");
                $zn->write_history(_("Zone Created"), ACTION_CREATE, 
                                    $user->username, $zn->id);
            }
            break;

        default:
            die_hard(_("Invalid form action"));
            break;
            
        }

        return $res;
    }

//}}}
//{{{   form_success()

    /**
     *  This method displays a success page to the user so they know their action
     *  worked like charm.
     *
     *  @return DIVTag  $div    A div that contains a success message
     */
    function form_success()
    {

        $div = html_div("msuccess");
        $indlink = html_a(web_root()."index.php", _("[Return to Index]"));
        $zonlink = html_a(web_root()."zone.php",_("[Return to Zone List]"));
        $div->add(_("Zone successfully created!"),html_br());
        $div->add(container($indlink,$zonlink));
        
        return $div;

    }

//}}}

}

?>
