<?php

/* vim: set expandtab tabstop=4 shiftwidth=4 foldmethod=marker: */
// $HeadURL: https://svn.osuosl.org/public/maintain2/trunk/modules/maintain_search/maintain_search.php $
// +------------------------------------------------------------------------+
// | Maintain - DNS/DHCP Managment Tool                                     |
// +------------------------------------------------------------------------+
// | Copyright (c) 2004 Open Source University                              |
// +------------------------------------------------------------------------+
// | This program is free software; you can redistribute it and/or          |
// | modify it under the terms of the GNU General Public License            |
// | as published by the Free Software Foundation; either version 2         |
// | of the License, or (at your option) any later version.                 |
// |                                                                        |
// | This program is distributed in the hope that it will be useful,        |
// | but WITHOUT ANY WARRANTY; without even the implied warranty of         |
// | MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          |
// | GNU General Public License for more details.                           |
// |                                                                        |
// | You should have received a copy of the GNU General Public License      |
// | along with this program; if not, write to the Free Software            |
// | Foundation, Inc., 59 Temple Place - Suite 330,                         |
// | Boston, MA  02111-1307, USA.                                           |
// +------------------------------------------------------------------------+
// | Author(s): Zack Bartel <bartel@osuosl.org>                             |
// +------------------------------------------------------------------------+
// $Id: maintain_search.php 3927 2006-02-20 15:47:47Z bartel $

/**
  * maintain_search.php - contains the {@link Maintain_Search} class
  *
  * @author $Author: bartel $
  * @filesource
  * @package    modules
  * @subpackage maintain_search
  */

require_once MODDIR.'/core_module.php';

define ("MAXRESULTS", 1000);

/**
  * This is the 'Maintain_Search' module for Maintain, providing core search functionality
3B
  *
  * @package    modules
  * @subpackage maintain_search
  */
class Maintain_Search extends Core_Module {

//{{{ vars
    /* Basically a lookup table for fields in our objects */
    var $fields = array(
                        "host"   => array("Hostname" => "name", 
                                        "Domain" => "domain", 
                                        "IP address" => "ip", 
                                        "Hardware Address" => "ha", 
                                        "Workgroup"  => "workgroup",
                                        "Host TTL"   => "ttl",
                                        "Status"     => "enabled",
                                        "Expire Date"=> "expire",
                                        "Operating System" => "os",
                                        "Location"  => "location",
                                        "Serial Number" => "serial",
                                        "Purchase Date" => "purchase_date",
                                        "PO Number" =>     "po_number",
                                        "Warranty Date" => "warranty_date",
                                        "Owning Unit" => "owning_unit",
                                        "Other ID"    => "other_id",
                                        "User ID"     => "user_id",
                                        ),
                        "zone"  => array("Name" => "name",
                                        "Description" => "description",
                                        "Comment" => "comment",
                                        "Support Email" => "support_mail"
                                        ),
                        "domain"  => array("Name" => "name", 
                                        ),
                        "workgroup" => array("Name" => "name"
                                        ),
                        "subnet"    => array("Name"=> "name",
                                        "Subnet" => "subnet"
                                        ),
                        "mx" =>       array("MX Record" => "name", 
                                        "Server" => "server", 
                                        ),
                        "pointer" =>  array("Hostname" => "hostname",
                                        "Type"    =>  "type"
                                        ),
                        "cname" => array("Name" => "name", 
                                        "CNAME Server" => "server", 
                                        ),
                        "user" => array("Username" => "username", 
                                        "E-Mail" => "email"),
                        "object_history" => array("User" => "username",
                                        "Comment" => "comment",
                                        "Summary" => "summary",
                                        "Object Type" => "object_type"
                                        )
                    );

    /* The searchable objects */
    var $objects = array("host"     => "Host",
                        "zone"      => "Zone",
                        "domain"    => "Domain",
                        "workgroup" => "Workgroup",
                        "subnet"    => "Subnet",
                        "mx"        => "MX Record",
                        "pointer"   => "Pointer",
                        "cname"     => "CNAME",
                        "user"      => "Users",
                        "object_history" => "History");
    var $results = array(); 

//}}}

/* Method Overrides */
// {{{ _install()

    function _install()
    {
        return true;
    }

// }}}
// {{{ _uninstall()

    function _uninstall()
    {

        return true;

    }

// }}}
// {{{ register_hooks()
    /**
      * Register the search module with the module hooks it should reply to
      */
    function register_hooks()
    {
        Module::add_filter('navlinks', 'makeNavLinks');
        Module::add_action('search', 'searchIsSearchable');
    }
// }}}

/* Maintain_Search methods */
// {{{ searchIsSearchable()

    /**
      * returns true if the current module provides a search function at all.
      * false otherwise.
      *
      * @access public
      * @return boolean searchable?
      */
    function searchIsSearchable()
    {
        return true;
    }

// }}}
// {{{ searchGetObjects()

    /**
      * This function will return an array of Strings (human-readable
      * object names) that the module will support search for.
      * False otherwise.
      *
      * @access public
      * @return mixed array of object names, or false.
      */
    function searchGetObjects()
    {
        return $this->objects;
    }
    
// }}}
// {{{ searchGetFields(.)

    /**
      * given an object name (as defined by searchGetObjects()), this function
      * returns an array of fields that are searchable through the search form
      * for the given object
      *
      * @param object name
      * @return mixed array of searchable fields or false
      */
    function searchGetFields($objName)
    {
//crap
//        if (is_callable(array($objName, "get_history")))
//            $this->fields[$objName] = array_merge($this->fields[$objName], array("History" => "history"));
        return $this->fields[$objName];
    }
      
// }}}
// {{{ searchGetAcceptableTypes(.)

    /**
      * given an object name (as defined by searchGetObjects()),
      * returns an array of acceptable search types for these.
      * (e.g. Exact, Fuzzy, Regex...)
      * false otherwise.
      *
      * @param object name
      * @return mixed array of search types or false
      */
    function searchGetAcceptableTypes($objName)
    {
        // TODO This should probably be defined somewhere
        if ($objName == "host")
            return array("Exact", "Fuzzy", "Regex");
        else if ($objName == "object_history")
            return array("Fuzzy", "Regex");
        else
            return array("Exact", "Fuzzy");
    }
      
// }}}
// {{{ searchDisplayColumns()

    /**
      * Will return an array of the form (Display => backend) that will
      * be used for the result data list. "display" is the name displayed
      * to the user, backend will be used internally.
      * false otherwise.
      *
      * @param string object being searched for
      * @return mixed array of display columns or false
      */
    function searchDisplayColumns($object)
    {
        $ary = array();
        foreach ($this->fields[$object] as $var=>$val)
            $ary[$var] = $val;
        return array_slice($ary, 0, 5);
    }
      
// }}}
// {{{ search(.)

    /**
      * executes an actual search on the objects searchable by this
      * module.
      * false otherwise.
      *
      * @param search query string
      * @param object name of object type to search for
      * @param attribute fields of objects to search in
      * @param type of search to be executed as defined by searchGetAcceptableTypes
      * @return mixed array of display columns or false
      */
    function search($query, $objname, $fields, $type)
    {
        global $user;
        //Don't think we ever want to do this. With an updgraded db the defaults are not set so when doing
        //An exact search it will fill out default data and not match the *old* db entries. However, new
        //data should work
     //   if (class_exists($objname)) {
     //       $obj = new $objname;
      //  } else
            $obj = DB_DataObject::Factory($objname);

        /* Set search limit */
        $obj->limit(MAXRESULTS);
    
        /* Handle different types of searches and special fields (unfortuneatley this hack has to be) */
        foreach ($fields as $field)  { 
            /* check for special cases */
            /* this is where most of the UGLY special cases go */

            //If it's a hardware address remove the :
            if (!strcasecmp($field, "ha"))
                $query = str_replace(":", "", $query);
            else if (!strcasecmp($field, "enabled"))
                if (!strcasecmp($query, "on") || !strcasecmp($query, "enable") ||
                    !strcasecmp($query, "enabled") || !strcasecmp($query, "true") ||
                    !strcasecmp($query, "yes") || !strcasecmp($query, "1") || $query == 1)
                        $query = 1;
                else
                    $query = 0;

            switch ($field) {

            /* if it is an ip we need to convert to int and check a range */
            case "ip":
                $high = $this->ip_range_high($query);
                $low = $this->ip_range_low($query);
                $obj->whereAdd($field. " <= ". IP::ip2int($high) . " AND ". $field . " >= ".IP::ip2int($low));
            break;
            case "expire":
            case "warranty_date":
                $query = strtotime($query);                   
                $obj->whereAdd("$field <= ".$obj->escape($query));
                $obj->whereAdd("$field > 0");
            break;
            case "domain":
            case "workgroup":
                //Generic case for foreign key obj searches
    
                if (strcasecmp($objname, "host"))   //If it's not Host::Deomain we don't want it
                    break;
                //Need to JOIN the correct domain object with this host object
                $dbdo = new $field();
                $dbdo->selectAdd();
                if (!strcasecmp($type, "exact"))    //If it is an "Exact" Search
                    $dbdo->setName($query);
                else if (!strcasecmp($type, "regexp"))
                    $dbdo->whereAdd("$field.name REGEXP '".$dbdo->escape($query)."'");
                else
                    $dbdo->whereAdd("$field.name LIKE '%".$dbdo->escape($query)."%'");
            
                $obj->selectAdd();
                $obj->selectAdd("host.id");
                $obj->joinAdd($dbdo);
            break;
            case "username":
                if (!strcasecmp($type, "exact"))    //If it is an "Exact" Search
                    $obj->whereAdd("user.username = '".$obj->escape($query)."'");
                else if (!strcasecmp($type, "regexp"))
                    $obj->whereAdd("user.username REGEXP '".$obj->escape($query)."'");
                else
                    $obj->whereAdd("user.username LIKE '%".$obj->escape($query)."%'");
            break;
            /* Most cases */
            default:
                /* Yes, it really is that easy */
                if (!strcasecmp($type, "fuzzy"))
                    $obj->whereAdd(/*$obj->tableName().".".*/$field. " LIKE '%". $obj->escape($query). "%'");
                else if (!strcasecmp($type, "regex"))
                    $obj->whereAdd($field. " REGEXP '". $obj->escape($query). "'");
                else    {
                    $method = "set" . $field;
                    $obj->$method($query);
                }
            break;
            }
        }

        /* Need to restrict search results to the current zone */

        /* Most of the time this will suffice */
        $obj->zone = $_SESSION['zone'];

        /* For many-to-manys (zone_workgroup) */
        $is_many_many = false;
        $zmm = DB_DataObject::Factory('zone_'.$objname);
        /* if there is a zone_$obj table then add a join */
        if (!PEAR::isError($zmm))   {
            $is_many_many = true;
            $zmm->zone = $_SESSION['zone'];
            $zmm->joinAdd($obj);
            $obj = $zmm;
        }
        $obj->find();
        //Get the unique identifier of the object (Host::id, User::username?)
        $keys = $obj->keys();
        $unique = $keys[0];
        while ($obj->fetch())   {
            if ((isset($obj->$objname) || !strcasecmp($objname, "user")) && $is_many_many )  {
                if (!strcasecmp($objname, "user"))
                    $this->results[] = $obj->username;
                else
                    $this->results[] = $obj->$objname;
            }
            else    {
                $this->results[] = $obj->$unique;
            }
        }
        return $this->results;
    }
      
// }}}
// {{{ searchFetch(.)

    /**
      * Returns a field value for a given object and UID (false otherwise).
      * The information will be used to populate the rows of the search
      * result table
      *
      * @param object name of object type to search for
      * @param unique identifier of object to be queried
      * @param field name of attribute to be returned
      * @return string representation of the queried object attribute,
      * false otherwise.
      */
    function searchFetch($objname, $uid, $field)
    {
        /* Method: 
            1. Decide what kind of object it is and how to instatiate it
            2. Figure out which method or field to get data from
            3. Uhh return it?
        */

        /* Is it an extended dbdo "Maintain main object" */
        $result;
        if (class_exists($objname)) {
            $o = new $objname($uid);
        }
        /* Or else use DB_DO's factory method */
        else    {
            $o = DB_DataObject::Factory($objname);
            if (!$o) return ;
            /* Grab the first key */
            //$key = array_shift($o->keys());
            //$o->$key = $uid;
            $o->get($uid);
            /* grab te object */
            //if (!$o->find()) return ;  
            //$o->fetch();
        } 

        /* Check to see if it's the history column */
        if ($field == "history")    {
            //return html_a($_SERVER['SCRIPT_NAME']."?action=history&amp;object_type={$objname}&amp;id={$uid}", "History");
            return html_a($_SERVER['SCRIPT_NAME']."?action=submit&amp;Search={$o->id}&amp;Object=maintain_search::object_history&amp;Field=object_id&amp;Type=Fuzzy&amp;_object_type=".$objname, "Show History");
        }
        
        /* if the class exists  and there is a get method */
        if (is_callable(array($objname, 'get'.$field))) {
            $method = "get".$field;
            $result = $o->$method();
        }   
        /* is the "$field" callable (method) */
        else if (is_callable(array($objname, $field)))  {
            $result = $o->$field();
        }
        /* If the class exists and there is a field */
        else if (isset($o->$field))   {
            $result = $o->$field;
        }

        /* Is there a htdoc to handle this? */
        if (file_exists($objname.".php"))   {
            return html_a(web_root().$objname.".php?action=edit&". (strcasecmp($objname, "user") ? "id=" : "username=") . $uid, $result);
        }
        /* Yay for special cases! */
        else if ($objname == "object_history")  {
            /* Make sure the object has not been deleted */
            if (class_exists($o->object_type))
                $daotest = new $o->object_type();
            else
                $daotest = DB_DataObject::Factory($o->object_type);
            if (method_exists($daotest, "get") && !$daotest->get($o->object_id))  {//is_callable(array($daotest, "get")))   {
                /* If it's not there anymore we can't link to it */
                return $result;
            }
            return html_a(web_root().$o->object_type.".php?action=edit&id=".$o->object_id, $result); 
        }
    }
      
// }}}


/* Private Module Methods */   
//{{{ makeNavLinks() (was: getNavBar)
    function makeNavLinks($menu)
    {
        global $images;
        $search_img = html_img($images[IMG_SEARCH]);
        $link = html_a(web_root()."search.php",container($search_img,_("Search")));
        $menu->add($link);
        
        return $menu;
    }
//}}}
//{{{   ip_range_low(.)
    /*
     *  helper function takes a dotted quad ip and change any blank
     *  or * into 0s
     *
     *  @param String Dotted quad IP address optionally with *s or missing octets
     *  @return String Dotted quad IP with 0 instead of * or nothing
     */
    function ip_range_low($ip)
    {
        $dq = explode(".", $ip);

        //IPv6 what?   
        for ($i = 0; $i < 4; $i++)
            if ($dq[$i] == '*' || !isset($dq[$i]))
                $dq[$i] = 0;
        return implode(".", $dq);
    }
//}}}
//{{{   ip_range_high(.)
    /*
     *  helper function takes a dotted quad ip and change any blank
     *  or * into 255s
     *
     *  @param String Dotted quad IP address optionally with *s or missing octets
     *  @return String Dotted quad IP with 255 instead of * or nothing
     */
    function ip_range_high($ip)
    {
        $dq = explode(".", $ip);

        //IPv6 what?   
        for ($i = 0; $i < 4; $i++)
            if ($dq[$i] == '*' || !isset($dq[$i]))
                $dq[$i] = 255;
        return implode(".", $dq);
    }
//}}}
}
?>
